@@ -15,9 +15,9 @@ class Node:
     def add_child(self, node):
         self.subclasses.append(node)
 
-def build_object_tree(objects):
+def build_object_tree(parser):
     # reorder objects so that parent classes come first ...
-    objects = objects[:]
+    objects = parser.objects[:]
     pos = 0
     while pos < len(objects):
         parent = objects[pos].parent
@@ -37,6 +37,33 @@ def build_object_tree(objects):
         parent_node.add_child(node)
         nodes[node.name] = node
 
+    if parser.interfaces:
+        interfaces = Node('gobject.GInterface')
+        root.add_child(interfaces)
+        nodes[interfaces.name] = interfaces
+        for obj_def in parser.interfaces:
+            node = Node(obj_def.c_name)
+            interfaces.add_child(node)
+            nodes[node.name] = node
+
+    if parser.boxes:
+        boxed = Node('gobject.GBoxed')
+        root.add_child(boxed)
+        nodes[boxed.name] = boxed
+        for obj_def in parser.boxes:
+            node = Node(obj_def.c_name)
+            boxed.add_child(node)
+            nodes[node.name] = node
+
+    if parser.pointers:
+        pointers = Node('gobject.GPointer')
+        root.add_child(pointers)
+        nodes[pointers.name] = pointers
+        for obj_def in parser.pointers:
+            node = Node(obj_def.c_name)
+            pointers.add_child(node)
+            nodes[node.name] = node
+
     return root
 
 class DocWriter:
@@ -80,11 +107,11 @@ class DocWriter:
     def output_docs(self, output_prefix):
         files = []
 
-        # class heirachy
-        heirachy = build_object_tree(self.parser.objects)
-        filename = self.create_filename('heirachy', output_prefix)
+        # class hierarchy
+        hierarchy = build_object_tree(self.parser)
+        filename = self.create_filename('hierarchy', output_prefix)
         fp = open(filename, 'w')
-        self.write_full_heirachy(heirachy, fp)
+        self.write_full_hierarchy(hierarchy, fp)
         fp.close()
 
 	obj_defs = self.parser.objects + self.parser.interfaces + \
@@ -117,7 +144,7 @@ class DocWriter:
         self.write_synopsis(obj_def, fp)
         self.close_section(fp)
 
-	# construct the inheritence heirachy ...
+	# construct the inheritence hierarchy ...
 	ancestry = [ (obj_def.c_name, obj_def.implements) ]
         try:
             parent = obj_def.parent
@@ -133,7 +160,7 @@ class DocWriter:
             pass
         ancestry.reverse()
         self.write_heading('Ancestry', fp)
-        self.write_heirachy(obj_def.c_name, ancestry, fp)
+        self.write_hierarchy(obj_def.c_name, ancestry, fp)
         self.close_section(fp)
 
         constructor = self.parser.find_constructor(obj_def, self.overrides)
@@ -211,7 +238,7 @@ class DocWriter:
     def create_toc_filename(self, output_prefix):
         return self.create_filename(self, 'docs', output_prefix)
 
-    def write_full_heirachy(self, heirachy, fp):
+    def write_full_hierarchy(self, hierarchy, fp):
         def handle_node(node, fp, indent=''):
             for child in node.subclasses:
                 fp.write(indent + node.name)
@@ -222,7 +249,7 @@ class DocWriter:
                 else:
                     fp.write('\n')
                 handle_node(child, fp, indent + '  ')
-        handle_node(heirachy, fp)
+        handle_node(hierarchy, fp)
 
     # these need to handle default args ...
     def create_constructor_prototype(self, func_def):
@@ -269,7 +296,7 @@ class DocWriter:
             prototype = self.create_method_prototype(meth)
             fp.write('    def %s\n' % prototype)
 
-    def write_heirachy(self, obj_name, ancestry, fp):
+    def write_hierarchy(self, obj_name, ancestry, fp):
         indent = ''
         for name, interfaces in ancestry:
             fp.write(indent + '+-- ' + name)
@@ -333,7 +360,7 @@ class DocbookDocWriter(DocWriter):
             return stem + '.sgml'
     def create_toc_filename(self, output_prefix):
         if self.use_xml:
-            return self.create_filename('docs', output_prefix)
+            return self.create_filename('classes', output_prefix)
         else:
             return self.create_filename('docs', output_prefix)
 
@@ -401,12 +428,15 @@ class DocbookDocWriter(DocWriter):
         # fall through through
         return '<literal>' + match.group(1) + '</literal>'
     
-    def reformat_text(self, text):
+    def reformat_text(self, text, singleline=0):
         # replace special strings ...
         text = self.__function_pat.sub(self.__format_function, text)
         text = self.__parameter_pat.sub(self.__format_param, text)
         text = self.__constant_pat.sub(self.__format_const, text)
         text = self.__symbol_pat.sub(self.__format_symbol, text)
+
+        # don't bother with <para> expansion for single line text.
+        if singleline: return text
         
         lines = string.split(string.strip(text), '\n')
         for index in range(len(lines)):
@@ -417,8 +447,8 @@ class DocbookDocWriter(DocWriter):
         lines.append('</para>')
         return string.join(lines, '\n')
 
-    # write out heirachy
-    def write_full_heirachy(self, heirachy, fp):
+    # write out hierarchy
+    def write_full_hierarchy(self, hierarchy, fp):
         def handle_node(node, fp, indent=''):
             if node.name:
                 fp.write('%s<link linkend="%s">%s</link>' %
@@ -441,13 +471,17 @@ class DocbookDocWriter(DocWriter):
                                  cmp(self.pyname(a.name), self.pyname(b.name)))
             for child in node.subclasses:
                 handle_node(child, fp, indent)
+        if self.use_xml:
+            fp.write('<?xml version="1.0" standalone="no"?>\n')
+            fp.write('<!DOCTYPE synopsis PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"\n')
+            fp.write('    "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">\n')
         fp.write('<synopsis>')
-        handle_node(heirachy, fp)
+        handle_node(hierarchy, fp)
         fp.write('</synopsis>\n')
 
     # these need to handle default args ...
     def create_constructor_prototype(self, func_def):
-        sgml = [ '  <constructorsynopsis language="python">\n']
+        sgml = [ '<constructorsynopsis language="python">\n']
         sgml.append('    <methodname>__init__</methodname>\n')
         for type, name, dflt, null in func_def.params:
             sgml.append('    <methodparam><parameter>')
@@ -460,10 +494,10 @@ class DocbookDocWriter(DocWriter):
             sgml.append('</methodparam>\n')
         if not func_def.params:
             sgml.append('    <methodparam></methodparam>')
-        sgml.append('  </constructorsynopsis>\n')
+        sgml.append('  </constructorsynopsis>')
         return string.join(sgml, '')
     def create_function_prototype(self, func_def):
-        sgml = [ '  <funcsynopsis language="python">\n    <funcprototype>\n']
+        sgml = [ '<funcsynopsis language="python">\n    <funcprototype>\n']
         sgml.append('      <funcdef><function>')
         sgml.append(func_def.name)
         sgml.append('</function></funcdef>\n')
@@ -478,10 +512,10 @@ class DocbookDocWriter(DocWriter):
             sgml.append('</paramdef>\n')
         if not func_def.params:
             sgml.append('      <paramdef></paramdef')
-        sgml.append('    </funcprototype>\n  </funcsynopsis>\n')
+        sgml.append('    </funcprototype>\n  </funcsynopsis>')
         return string.join(sgml, '')
     def create_method_prototype(self, meth_def, addlink=0):
-        sgml = [ '  <methodsynopsis language="python">\n']
+        sgml = [ '<methodsynopsis language="python">\n']
         sgml.append('    <methodname>')
         if addlink:
             sgml.append('<link linkend="%s">' % self.make_method_ref(meth_def))
@@ -500,14 +534,14 @@ class DocbookDocWriter(DocWriter):
             sgml.append('</methodparam>\n')
         if not meth_def.params:
             sgml.append('    <methodparam></methodparam>')
-        sgml.append('  </methodsynopsis>\n')
+        sgml.append('  </methodsynopsis>')
         return string.join(sgml, '')
     
     def write_class_header(self, obj_name, fp):
-        #if self.use_xml:
-        #    fp.write('<?xml version="1.0" standalone="no"?>\n')
-        #    fp.write('<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"\n')
-        #    fp.write('    "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">\n')
+        if self.use_xml:
+            fp.write('<?xml version="1.0" standalone="no"?>\n')
+            fp.write('<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"\n')
+            fp.write('    "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">\n')
         fp.write('<refentry id="' + self.make_class_ref(obj_name) + '">\n')
         fp.write('  <refmeta>\n')
         fp.write('    <refentrytitle>%s</refentrytitle>\n'
@@ -541,18 +575,27 @@ class DocbookDocWriter(DocWriter):
                 fp.write('  <ooclass><classname><link linkend="%s">%s'
                          '</link></classname></ooclass>\n'
                          % (self.make_class_ref(base), self.pyname(base)))
+        elif isinstance(obj_def, definitions.InterfaceDef):
+            fp.write('  <ooclass><classname>gobject.GInterface'
+                     '</classname></ooclass>\n')
+        elif isinstance(obj_def, definitions.BoxedDef):
+            fp.write('  <ooclass><classname>gobject.GBoxed'
+                     '</classname></ooclass>\n')
+        elif isinstance(obj_def, definitions.PointerDef):
+            fp.write('  <ooclass><classname>gobject.GPointer'
+                     '</classname></ooclass>\n')
 
         constructor = self.parser.find_constructor(obj_def, self.overrides)
         if constructor:
-            fp.write(self.create_constructor_prototype(constructor))
+            fp.write('%s\n' % self.create_constructor_prototype(constructor))
         methods = self.parser.find_methods(obj_def)
         methods = filter(lambda meth, self=self:
                          not self.overrides.is_ignored(meth.c_name), methods)
         for meth in methods:
-            fp.write(self.create_method_prototype(meth, addlink=1))
+            fp.write('%s\n' % self.create_method_prototype(meth, addlink=1))
         fp.write('</classsynopsis>\n\n')
 
-    def write_heirachy(self, obj_name, ancestry, fp):
+    def write_hierarchy(self, obj_name, ancestry, fp):
         fp.write('<synopsis>')
         indent = ''
         for name, interfaces in ancestry:
@@ -572,38 +615,37 @@ class DocbookDocWriter(DocWriter):
             indent = indent + '  '
         fp.write('</synopsis>\n\n')
 
-    def write_constructor(self, func_def, func_doc, fp):
-        prototype = self.create_constructor_prototype(func_def)
-        fp.write(prototype + '\n')
-        varlist_started = 0
-        for type, name, dflt, null in func_def.params:
-            if not varlist_started:
-                fp.write('  <variablelist>\n')
-                varlist_started = 1
+    def write_params(self, params, ret, func_doc, fp):
+        if not params and (not ret or ret == 'none'):
+            return
+        fp.write('  <variablelist>\n')
+        for type, name, dflt, null in params:
             if func_doc:
                 descr = string.strip(func_doc.get_param_description(name))
             else:
                 descr = 'a ' + type
             fp.write('    <varlistentry>\n')
-            fp.write('      <term><parameter>'+ name +'</parameter></term>\n')
-            fp.write('      <listitem>' + self.reformat_text(descr) +
-                     '</listitem>\n')
+            fp.write('      <term><parameter>%s</parameter>&nbsp;:</term>\n' % name)
+            fp.write('      <listitem><simpara>%s</simpara></listitem>\n' %
+                     self.reformat_text(descr, singleline=1))
             fp.write('    </varlistentry>\n')
-        if func_def.ret and func_def.ret != 'none':
-            if not varlist_started:
-                fp.write('  <variablelist>\n')
-                varlist_started = 1
+        if ret and ret != 'none':
             if func_doc and func_doc.ret:
                 descr = string.strip(func_doc.ret)
             else:
-                descr = 'a ' + func_def.ret
+                descr = 'a ' + ret
             fp.write('    <varlistentry>\n')
-            fp.write('      <term>Returns:</term>\n')
-            fp.write('      <listitem>' + self.reformat_text(descr) +
-                     '</listitem>\n')
+            fp.write('      <term><emphasis>Returns</emphasis>&nbsp;:</term>\n')
+            fp.write('      <listitem><simpara>%s</simpara></listitem>\n' %
+                     self.reformat_text(descr, singleline=1))
             fp.write('    </varlistentry>\n')
-        if varlist_started:
-            fp.write('  </variablelist>\n')
+        fp.write('  </variablelist>\n')
+
+    def write_constructor(self, func_def, func_doc, fp):
+        prototype = self.create_constructor_prototype(func_def)
+        fp.write('<programlisting>%s</programlisting>\n' % prototype)
+        self.write_params(func_def.params, func_def.ret, func_doc, fp)
+
         if func_doc and func_doc.description:
             fp.write(self.reformat_text(func_doc.description))
         fp.write('\n\n\n')
@@ -613,36 +655,8 @@ class DocbookDocWriter(DocWriter):
         fp.write('    <title>' + self.pyname(meth_def.of_object) + '.' +
                  meth_def.name + '</title>\n\n')
         prototype = self.create_method_prototype(meth_def)
-        fp.write(prototype + '\n')
-        varlist_started = 0
-        for type, name, dflt, null in meth_def.params:
-            if not varlist_started:
-                fp.write('  <variablelist>\n')
-                varlist_started = 1
-            if func_doc:
-                descr = string.strip(func_doc.get_param_description(name))
-            else:
-                descr = 'a ' + type
-            fp.write('    <varlistentry>\n')
-            fp.write('      <term><parameter>'+ name +'</parameter></term>\n')
-            fp.write('      <listitem>' + self.reformat_text(descr) +
-                     '</listitem>\n')
-            fp.write('    </varlistentry>\n')
-        if meth_def.ret and meth_def.ret != 'none': 
-            if not varlist_started:
-                fp.write('  <variablelist>\n')
-                varlist_started = 1
-            if func_doc and func_doc.ret:
-                descr = string.strip(func_doc.ret)
-            else:
-                descr = 'a ' + meth_def.ret
-            fp.write('    <varlistentry>\n')
-            fp.write('      <term>Returns:</term>\n')
-            fp.write('      <listitem>' + self.reformat_text(descr) +
-                     '</listitem>\n')
-            fp.write('    </varlistentry>\n')
-        if varlist_started:
-            fp.write('  </variablelist>\n')
+        fp.write('<programlisting>%s</programlisting>\n' % prototype)
+        self.write_params(meth_def.params, meth_def.ret, func_doc, fp)
         if func_doc and func_doc.description:
             fp.write(self.reformat_text(func_doc.description))
         fp.write('  </refsect2>\n\n\n')
@@ -651,25 +665,25 @@ class DocbookDocWriter(DocWriter):
         if self.use_xml:
             fp.write('<?xml version="1.0" standalone="no"?>\n')
             fp.write('<!DOCTYPE reference PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"\n')
-            fp.write('    "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd" [\n')
-            for filename, obj_def in files:
-                fp.write('  <!ENTITY ' + string.translate(obj_def.c_name,
-                                                          self.__transtable) +
-                         ' SYSTEM "' + filename + '" >\n')
-            fp.write(']>\n\n')
-
-            fp.write('<reference id="class-reference">\n')
-            fp.write('  <title>Class Documentation</title>\n')
-            for filename, obj_def in files:
-                fp.write('&' + string.translate(obj_def.c_name,
-                                                self.__transtable) + ';\n')
-
-            fp.write('</reference>\n')
-            #fp.write('<reference id="class-reference" xmlns:xi="http://www.w3.org/2001/XInclude">\n')
-            #fp.write('  <title>Class Reference</title>\n')
+            fp.write('    "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">\n')
             #for filename, obj_def in files:
-            #    fp.write('  <xi:include href="%s"/>\n' % filename)
+            #    fp.write('  <!ENTITY ' + string.translate(obj_def.c_name,
+            #                                              self.__transtable) +
+            #             ' SYSTEM "' + filename + '" >\n')
+            #fp.write(']>\n\n')
+
+            #fp.write('<reference id="class-reference">\n')
+            #fp.write('  <title>Class Documentation</title>\n')
+            #for filename, obj_def in files:
+            #    fp.write('&' + string.translate(obj_def.c_name,
+            #                                    self.__transtable) + ';\n')
             #fp.write('</reference>\n')
+            
+            fp.write('<reference id="class-reference" xmlns:xi="http://www.w3.org/2001/XInclude">\n')
+            fp.write('  <title>Class Reference</title>\n')
+            for filename, obj_def in files:
+                fp.write('  <xi:include href="%s"/>\n' % filename)
+            fp.write('</reference>\n')
         else:
             fp.write('<!DOCTYPE article PUBLIC "-//OASIS//DTD DocBook V4.1.2//EN" [\n')
             for filename, obj_def in files:
@@ -689,8 +703,8 @@ class DocbookDocWriter(DocWriter):
             fp.write('    </authorgroup>\n')
             fp.write('  </bookinfo>\n\n')
 
-            fp.write('  <chapter id="class-heirachy">\n')
-            fp.write('    <title>Class Heirachy</title>\n')
+            fp.write('  <chapter id="class-hierarchy">\n')
+            fp.write('    <title>Class Hierarchy</title>\n')
             fp.write('    <para>Not done yet</para>\n')
             fp.write('  </chapter>\n\n')
 
