@@ -9,6 +9,7 @@
 /* include this first, before NO_IMPORT_PYGOBJECT is defined */
 #include <pygobject.h>
 #include "pygtk-private.h"
+#include <pyerrors.h>
 
 void _pygtk_register_boxed_types(PyObject *moddict);
 void pygtk_register_classes(PyObject *d);
@@ -25,6 +26,33 @@ static struct _PyGtk_FunctionStruct functions = {
     &PyGdkAtom_Type,  PyGdkAtom_New,
 };
 
+static void
+sink_gtkobject(GObject *object)
+{
+    if (GTK_OBJECT_FLOATING(object)) {
+	g_object_ref(object);
+	gtk_object_sink(GTK_OBJECT(object));
+    }
+}
+
+static gboolean
+python_do_pending_calls(gpointer data)
+{
+    gboolean quit = FALSE;
+
+    pyg_block_threads();
+    if (PyErr_CheckSignals() == -1) {
+	PyErr_SetNone(PyExc_KeyboardInterrupt);
+	quit = TRUE;
+    }
+    pyg_unblock_threads();
+
+    if (quit)
+	gtk_main_quit();
+    
+    return TRUE;
+}
+
 DL_EXPORT(void)
 init_gtk(void)
 {
@@ -39,6 +67,8 @@ init_gtk(void)
     init_pygobject();
     g_assert(pygobject_register_class != NULL);
 
+    pygobject_register_sinkfunc(GTK_TYPE_OBJECT, sink_gtkobject);
+
     /* set the default python encoding to utf-8 */
     PyUnicode_SetDefaultEncoding("utf-8");
 
@@ -142,5 +172,6 @@ init_gtk(void)
 
     if (PyErr_Occurred())
 	Py_FatalError("can't initialise module _gtk");
-
+    
+    gtk_timeout_add(100, python_do_pending_calls, NULL);
 }
