@@ -24,6 +24,28 @@ GType PY_TYPE_OBJECT;
 
 void  pyg_destroy_notify (gpointer user_data);
 
+typedef struct {
+    PyObject_HEAD
+    GParamSpec *pspec;
+} PyGParamSpec;
+extern PyTypeObject PyGParamSpec_Type;
+PyObject *pyg_param_spec_new(GParamSpec *pspec);
+
+#define PyGParamSpec_Check(v) (PyObject_TypeCheck(v, &PyGParamSpec_Type))
+#define PyGParamSpec_Get(v) (((PyGParamSpec *)v)->pspec)
+
+typedef struct {
+    PyObject_HEAD
+    GMainLoop *loop;
+} PyGMainLoop;
+extern PyTypeObject PyGMainLoop_Type;
+
+typedef struct {
+    PyObject_HEAD
+    GMainContext *context;
+} PyGMainContext;
+extern PyTypeObject PyGMainContext_Type;
+
 /* from pygtype.h */
 extern PyTypeObject PyGTypeWrapper_Type;
 
@@ -40,7 +62,15 @@ void      pyg_register_boxed_custom(GType boxed_type,
 				    fromvaluefunc from_func,
 				    tovaluefunc to_func);
 int       pyg_value_from_pyobject(GValue *value, PyObject *obj);
-PyObject *pyg_value_as_pyobject(const GValue *value);
+PyObject *pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed);
+
+typedef struct _PyGClosure PyGClosure;
+struct _PyGClosure {
+    GClosure closure;
+    PyObject *callback;
+    PyObject *extra_args; /* tuple of extra args to pass to callback */
+    PyObject *swap_data; /* other object for gtk_signal_connect_object */
+};
 
 GClosure *pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data);
 GClosure *pyg_signal_class_closure_get(void);
@@ -58,7 +88,9 @@ void          pygobject_register_class   (PyObject *dict,
 void          pygobject_register_wrapper (PyObject *self);
 PyObject *    pygobject_new              (GObject *obj);
 PyTypeObject *pygobject_lookup_class     (GType gtype);
-
+void          pygobject_watch_closure    (PyObject *self, GClosure *closure);
+void          pygobject_register_sinkfunc(GType type,
+					  void (* sinkfunc)(GObject *object));
 
 /* from pygboxed.c */
 extern PyTypeObject PyGBoxed_Type;
