@@ -5,11 +5,10 @@ static const gchar *pygobject_class_id     = "PyGObject::class";
 static GQuark       pygobject_class_key    = 0;
 static const gchar *pygobject_wrapper_id   = "PyGObject::wrapper";
 static GQuark       pygobject_wrapper_key  = 0;
-static const gchar *pygobject_ownedref_id  = "PyGObject::ownedref";
-static GQuark       pygobject_ownedref_key = 0;
 
 static void pygobject_dealloc(PyGObject *self);
 static int  pygobject_traverse(PyGObject *self, visitproc visit, void *arg);
+static int  pygobject_clear(PyGObject *self);
 
 /* -------------- class <-> wrapper manipulation --------------- */
 
@@ -35,6 +34,19 @@ sink_object(GObject *obj)
     }
 }
 
+/**
+ * pygobject_register_sinkfunc:
+ * type: the GType the sink function applies to.
+ * sinkfunc: a function to remove the floating reference on an object.
+ *
+ * As Python handles reference counting for us, the "floating
+ * reference" code in GTK is not all that useful.  In fact, it can
+ * cause leaks.  For this reason, PyGTK removes the floating
+ * references on objects on construction.
+ *
+ * The sinkfunc should be able to remove the floating reference on
+ * instances of the given type, or any subclasses.
+ */
 void
 pygobject_register_sinkfunc(GType type, void (* sinkfunc)(GObject *object))
 {
@@ -48,6 +60,19 @@ pygobject_register_sinkfunc(GType type, void (* sinkfunc)(GObject *object))
     g_array_append_val(sink_funcs, sf);
 }
 
+/**
+ * pygobject_register_class:
+ * @dict: the module dictionary.  A reference to the type will be stored here.
+ * @type_name: not used ?
+ * @gtype: the GType of the GObject subclass.
+ * @type: the Python type object for this wrapper.
+ * @bases: a tuple of Python type objects that are the bases of this type.
+ *
+ * This function is used to register a Python type as the wrapper for
+ * a particular GObject subclass.  It will also insert a reference to
+ * the wrapper class into the module dictionary passed as a reference,
+ * which simplifies initialisation.
+ */
 void
 pygobject_register_class(PyObject *dict, const gchar *type_name,
 			 GType gtype, PyTypeObject *type,
@@ -72,6 +97,7 @@ pygobject_register_class(PyObject *dict, const gchar *type_name,
 
     type->tp_dealloc  = (destructor)pygobject_dealloc;
     type->tp_traverse = (traverseproc)pygobject_traverse;
+    type->tp_clear = (inquiry)pygobject_clear;
     type->tp_flags |= Py_TPFLAGS_HAVE_GC;
     type->tp_weaklistoffset = offsetof(PyGObject, weakreflist);
     type->tp_dictoffset = offsetof(PyGObject, inst_dict);
@@ -98,6 +124,16 @@ pygobject_register_class(PyObject *dict, const gchar *type_name,
     PyDict_SetItemString(dict, (char *)class_name, (PyObject *)type);
 }
 
+/**
+ * pygobject_register_wrapper:
+ * @self: the wrapper instance
+ *
+ * In the constructor of PyGTK wrappers, this function should be
+ * called after setting the obj member.  It will tie the wrapper
+ * instance to the GObject so that the same wrapper instance will
+ * always be used for this GObject instance.  It will also sink any
+ * floating references on the GObject.
+ */
 void
 pygobject_register_wrapper(PyObject *self)
 {
@@ -107,9 +143,23 @@ pygobject_register_wrapper(PyObject *self)
 	pygobject_wrapper_key=g_quark_from_static_string(pygobject_wrapper_id);
 
     sink_object(obj);
-    g_object_set_qdata(obj, pygobject_wrapper_key, self);
+    Py_INCREF(self);
+    g_object_set_qdata_full(obj, pygobject_wrapper_key, self,
+			    pyg_destroy_notify);
 }
 
+/**
+ * pygobject_lookup_class:
+ * @gtype: the GType of the GObject subclass.
+ *
+ * This function looks up the wrapper class used to represent
+ * instances of a GObject represented by @gtype.  If no wrapper class
+ * has been registered for the given GType, then the parent GType will
+ * be checked.  Since a wrapper has been registered for "GObject",
+ * this function will always succeed.
+ *
+ * Returns: The wrapper class for the GObject.
+ */
 PyTypeObject *
 pygobject_lookup_class(GType gtype)
 {
@@ -123,17 +173,24 @@ pygobject_lookup_class(GType gtype)
     return type;
 }
 
+/**
+ * pygobject_new:
+ * @obj: a GObject instance.
+ *
+ * This function gets a reference to a wrapper for the given GObject
+ * instance.  If a wrapper has already been created, a new reference
+ * to that wrapper will be returned.  Otherwise, a wrapper instance
+ * will be created.
+ *
+ * Returns: a reference to the wrapper for the GObject.
+ */
 PyObject *
 pygobject_new(GObject *obj)
 {
     PyGObject *self;
-    PyTypeObject *tp;
 
     if (!pygobject_wrapper_key)
 	pygobject_wrapper_key=g_quark_from_static_string(pygobject_wrapper_id);
-    if (!pygobject_ownedref_key)
-	pygobject_ownedref_key =
-	    g_quark_from_static_string(pygobject_ownedref_id);
 
     if (obj == NULL) {
 	Py_INCREF(Py_None);
@@ -141,34 +198,30 @@ pygobject_new(GObject *obj)
     }
 
     /* we already have a wrapper for this object -- return it. */
-    if ((self = (PyGObject *)g_object_get_qdata(obj, pygobject_wrapper_key))) {
-	/* if the GObject currently owns the wrapper reference ... */
-	if (self->hasref) {
-	    self->hasref = FALSE;
-	    g_object_steal_qdata(obj, pygobject_ownedref_key);
-	    g_object_ref(obj);
-	} else
-	    Py_INCREF(self);
-	return (PyObject *)self;
+    self = (PyGObject *)g_object_get_qdata(obj, pygobject_wrapper_key);
+    if (self != NULL) {
+	Py_INCREF(self);
+    } else {
+	/* create wrapper */
+	PyTypeObject *tp = pygobject_lookup_class(G_OBJECT_TYPE(obj));
+	self = PyObject_GC_New(PyGObject, tp);
+
+	if (self == NULL)
+	    return NULL;
+	self->obj = g_object_ref(obj);
+	sink_object(self->obj);
+
+	self->inst_dict = NULL;
+	self->weakreflist = NULL;
+	self->closures = NULL;
+	/* save wrapper pointer so we can access it later */
+	Py_INCREF(self);
+	g_object_set_qdata_full(obj, pygobject_wrapper_key, self,
+				pyg_destroy_notify);
+
+	PyObject_GC_Track((PyObject *)self);
     }
 
-    tp = pygobject_lookup_class(G_OBJECT_TYPE(obj));
-    self = PyObject_GC_New(PyGObject, tp);
-
-    if (self == NULL)
-	return NULL;
-    self->obj = g_object_ref(obj);
-    sink_object(self->obj);
-
-    self->hasref = FALSE;
-    self->inst_dict = NULL;
-    self->weakreflist = NULL;
-    self->closures = NULL;
-    /* save wrapper pointer so we can access it later */
-    g_object_set_qdata(obj, pygobject_wrapper_key, self);
-
-    PyObject_GC_Track((PyObject *)self);
-
     return (PyObject *)self;
 }
 
@@ -180,6 +233,18 @@ pygobject_unwatch_closure(gpointer data, GClosure *closure)
     self->closures = g_slist_remove (self->closures, closure);
 }
 
+/**
+ * pygobject_watch_closure:
+ * @self: a GObject wrapper instance
+ * @closure: a GClosure to watch
+ *
+ * Adds a closure to the list of watched closures for the wrapper.
+ * The closure must be one returned by pyg_closure_new().  When the
+ * cycle GC traverses the wrapper instance, it will enumerate the
+ * references to Python objects stored in watched closures.  If the
+ * cycle GC tells the wrapper to clear itself, the watched closures
+ * will be invalidated.
+ */
 void
 pygobject_watch_closure(PyObject *self, GClosure *closure)
 {
@@ -200,55 +265,25 @@ pygobject_watch_closure(PyObject *self, GClosure *closure)
 static void
 pygobject_dealloc(PyGObject *self)
 {
-    GObject *obj = self->obj;
     GSList *tmp;
 
-    if (!pygobject_ownedref_key)
-	pygobject_ownedref_key =
-	    g_quark_from_static_string(pygobject_ownedref_id);
-
-    /* save reference to python wrapper if there are still
-     * references to the GObject in such a way that it will be
-     * freed when the GObject is destroyed, so is the python
-     * wrapper, but if a python wrapper can be */
-    if (obj && obj->ref_count > 1) {
-	Py_INCREF(self); /* grab a reference on the wrapper */
-	self->hasref = TRUE;
-	g_object_set_qdata_full(obj, pygobject_ownedref_key,
-				self, pyg_destroy_notify);
-	
-	pyg_unblock_threads();
-	g_object_unref(obj);
-	pyg_block_threads();
-
-	/* we ref the type, so subtype_dealloc() doesn't kill off our
-         * instance's type. */
-	if (self->ob_type->tp_flags & Py_TPFLAGS_HEAPTYPE)
-	    Py_INCREF(self->ob_type);
-	
-#ifdef Py_TRACE_REFS
-	/* if we're tracing refs, set up the reflist again, as it was just
-	 * torn down */
-        _Py_NewReference((PyObject *) self);
-#endif
-
-	return;
-    }
-    if (obj && !self->hasref) { /* don't unref the GObject if it owns us */
-	pyg_unblock_threads();
-	g_object_unref(obj);
-	pyg_block_threads();
-    }
-
     PyObject_ClearWeakRefs((PyObject *)self);
 
     PyObject_GC_UnTrack((PyObject *)self);
 
+    if (self->obj) {
+	pyg_unblock_threads();
+	g_object_unref(self->obj);
+	pyg_block_threads();
+    }
+    self->obj = NULL;
+
     if (self->inst_dict) {
 	Py_DECREF(self->inst_dict);
     }
     self->inst_dict = NULL;
 
+    pyg_unblock_threads();
     tmp = self->closures;
     while (tmp) {
 	GClosure *closure = tmp->data;
@@ -259,9 +294,10 @@ pygobject_dealloc(PyGObject *self)
 	g_closure_invalidate(closure);
     }
     self->closures = NULL;
+    pyg_block_threads();
 
     /* the following causes problems with subclassed types */
-    /*self->ob_type->tp_free((PyObject *)self); */
+    /* self->ob_type->tp_free((PyObject *)self); */
     PyObject_GC_Del(self);
 }
 
@@ -313,6 +349,11 @@ pygobject_traverse(PyGObject *self, visitproc visit, void *arg)
 	if (closure->swap_data) ret = visit(closure->swap_data, arg);
 	if (ret != 0) return ret;
     }
+
+    if (self->obj && self->obj->ref_count == 1)
+	ret = visit((PyObject *)self, arg);
+    if (ret != 0) return ret;
+
     return 0;
 }
 
@@ -321,6 +362,12 @@ pygobject_clear(PyGObject *self)
 {
     GSList *tmp;
 
+    if (self->inst_dict) {
+	Py_DECREF(self->inst_dict);
+    }
+    self->inst_dict = NULL;
+
+    pyg_unblock_threads();
     tmp = self->closures;
     while (tmp) {
 	GClosure *closure = tmp->data;
@@ -330,9 +377,18 @@ pygobject_clear(PyGObject *self)
 	tmp = tmp->next;
 	g_closure_invalidate(closure);
     }
+    pyg_block_threads();
+
     if (self->closures != NULL)
 	g_message("invalidated all closures, but self->closures != NULL !");
 
+    if (self->obj) {
+	pyg_unblock_threads();
+	g_object_unref(self->obj);
+	pyg_block_threads();
+    }
+    self->obj = NULL;
+
     return 0;
 }
 
